/****************************************************************************
 *   PacketDecoder.h by (c) 2019 Marcio Teixeira                            *
 *                                                                          *
 *   This program is free software: you can redistribute it and/or modify   *
 *   it under the terms of the GNU General Public License as published by   *
 *   the Free Software Foundation, either version 3 of the License, or      *
 *   (at your option) any later version.                                    *
 *                                                                          *
 *   This program is distributed in the hope that it will be useful,        *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of         *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for more details.                           *
 *                                                                          *
 *   To view a copy of the GNU General Public License, go to the following  *
 *   location: <http://www.gnu.org/licenses/>.                              *
 ****************************************************************************/

#ifdef ARDUINO
#include "Arduino.h"
#endif

#include "lib/arduinoHSV/arduinoHSV.h"

typedef enum : uint8_t {
  BTN_NONE       = 0,
  BTN_ON         = 1,
  BTN_OFF        = 2,
  BTN_GROUP1_ON  = 3,
  BTN_GROUP1_OFF = 4,
  BTN_GROUP2_ON  = 5,
  BTN_GROUP2_OFF = 6,
  BTN_GROUP3_ON  = 7,
  BTN_GROUP3_OFF = 8,
  BTN_GROUP4_ON  = 9,
  BTN_GROUP4_OFF = 10,
  BTN_S_PLUS     = 11,
  BTN_S_MINUS    = 12,
  BTN_M_PLUS     = 13,
  BTN_BIGHTNESS  = 14,
  BTN_HUE        = 15      
} button_t;

typedef enum : uint8_t {
  MODE_SMOOTH_COLOR_FADE = 0,
  MODE_SMOOTH_WHITE_FADE = 1,
  MODE_SMOOTH_RGB_FADE   = 2,
  MODE_COLOR_CYCLE       = 3,
  MODE_RANDOM            = 4,
  MODE_RED_FADE_BLINK    = 5,
  MODE_GREEN_FADE_BLINK  = 6,
  MODE_BLUE_FADE_BLINK   = 7,
  MODE_SHOW_ALL_EFFECTS  = 8
} effect_t;
    
class PacketDecoder {
  protected:
    uint8_t hue;
    uint8_t brightness;
    uint8_t button;
    uint8_t effect;

  public:
    virtual void decode(uint8_t const* packet, size_t length) = 0;
    
    button_t getButton()     {return button & 0x0F;}
    uint8_t  getBrightness() {return brightness;}
    int      getHue()        {return map(hue, 0, 255, 0, 360);}
    void     getRgb(uint8_t &r, uint8_t &g, uint8_t &b) {
      hsv2rgb(hue, 255, brightness, r, g, b);
    }
};